@@ -1,7 +1,7 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
 ;;; Copyright © 2014, 2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
-;;; Copyright © 2014, 2015, 2016, 2017, 2018, 2020, 2021, 2022 Rutger van Beusekom <rutger@dezyne.org>
+;;; Copyright © 2014, 2015, 2016, 2017, 2018, 2020, 2021, 2022, 2023 Rutger van Beusekom <rutger@dezyne.org>
 ;;; Copyright © 2016, 2017, 2018, 2019, 2020 Rob Wieringa <rma.wieringa@gmail.com>
 ;;; Copyright © 2018 Filip Toman <filip.toman@verum.com>
 ;;;
@@ -65,6 +65,7 @@
             code:class-member?
             code:component-include
             code:component-port
+            code:data*
             code:declarative-or-imperative
             code:default-true
             code:defer-condition
@@ -156,12 +157,13 @@
 
 (define-method (code:interface-include (o <top>) source-file)
   (let* ((interfaces (ast:interface* o))
-         (files (filter (compose not
-                                 (cute equal? source-file <>)
-                                 ast:source-file)
-                        interfaces)))
-    (map (compose (cut make <file-name> #:name <>) code:file-name)
-         files)))
+         (interfaces (filter (compose not
+                                      (cute equal? source-file <>)
+                                      ast:source-file)
+                             interfaces))
+         (file-names (map code:file-name interfaces))
+         (file-names (delete-duplicates file-names)))
+    (map (cut make <file-name> #:name <>) file-names)))
 
 (define-method (code:interface-include (o <top>))
   (code:interface-include o (ast:source-file o)))
@@ -206,6 +208,9 @@
 (define-method (code:file-name (o <foreign>))
   (ast:full-name o))
 
+(define-method (code:file-name (o <import>))
+  (basename (.name o) ".dzn"))
+
 (define-method (code:file-name (o <ast>))
   (basename (ast:source-file o) ".dzn"))
 
@@ -371,14 +376,24 @@
 (define-method (code:return (o <on>))
   (code:return (code:trigger o)))
 
+(define (code:blocking? o)
+  (pair? (tree-collect-filter
+          (negate (disjoin (is? <imperative>)
+                           (is? <expression>)
+                           (is? <location>)))
+          (disjoin (is? <blocking>) (is? <blocking-compound>))
+          (parent o <model>))))
+
 (define-method (code:port-release o)
-  (if (null? (tree-collect-filter
-              (negate (disjoin (is? <imperative>)
-                               (is? <expression>)
-                               (is? <location>)))
-              (disjoin (is? <blocking>) (is? <blocking-compound>))
-              (parent o <model>))) '()
-              o))
+  (let ((trigger (and=> (parent o <on>)
+                        (compose car ast:trigger*))))
+    (and (or (not trigger)
+             (ast:requires? trigger)
+             (or (not (ast:equal? (.port o) (.port trigger)))
+                 (parent o <blocking>)
+                 (parent o <blocking-compound>)))
+         (code:blocking? o)
+         o)))
 
 (define-method (code:port-reply o)
   (let ((port (.port o)))
@@ -503,6 +518,9 @@
 (define-method (code:variable-name (o <argument>))
   o)
 
+(define-method (code:data* (o <root>))
+  (filter (negate ast:imported?) (ast:data* o)))
+
 
 ;;;
 ;;; Enum
@@ -538,14 +556,13 @@
   (filter (is? <enum>) (ast:type* (.behavior o))))
 
 (define-method (code:global-enum-definer (o <root>))
-  (filter (is? <enum>) (ast:type* o)))
+  (filter (conjoin (is? <enum>)
+                   (negate ast:imported?))
+          (ast:type* o)))
 
 (define-method (code:global-enum-definer (o <model>))
   (filter (is? <enum>) (ast:type* (parent o <root>))))
 
-(define-method (code:global-enum-definer (o <root>))
-  (filter (is? <enum>) (ast:type* o)))
-
 (define-method (code:enum-literal (o <enum-literal>))
   (append (code:type-name (.type o)) (list (.field o))))
 
