@@ -66,6 +66,9 @@
     (cond
      ((eq? event 'async)
       (flush-async pc '()))
+     ((eq? event 'defer)
+      (if (null? (.defer pc)) '()
+          (run-defer-event pc event)))
      ((eq? event 'external)
       (process-external pc))
      ((is-a? (%sut) <runtime:port>)
@@ -132,16 +135,24 @@ that PC has one more collaterally blocked coroutine on the same port."
        #f)))
 
   (define (run-label orig-pc label)
-    (let* ((sw-pc (switch-context orig-pc))
+    (let* ((orig-pc (clone orig-pc #:collateral-instance #f))
+           (sw-pc (switch-context orig-pc))
            (bob-pc (blocked-on-boundary-switch-context orig-pc)))
       (if (and (eq? orig-pc sw-pc)
                (eq? orig-pc bob-pc))
           (run-to-completion** orig-pc label)
-          (if (and (not (eq? orig-pc sw-pc))
-                   (not (provides-trigger? label)))
-              (run-to-completion sw-pc 'rtc)
-              (append (run-to-completion bob-pc 'rtc)
-                      (run-to-completion** orig-pc label))))))
+          (cond
+           ((and (not (eq? orig-pc sw-pc))
+                 (not (provides-trigger? label)))
+            (run-to-completion sw-pc 'rtc))
+           (else
+            (append (let* ((traces (run-to-completion bob-pc 'rtc))
+                           (flush traces
+                                  (partition (compose .running-defer? car)
+                                             traces)))
+                      (append traces
+                              (append-map run-flush flush)))
+                    (run-to-completion** orig-pc label)))))))
 
   (define (run-labels pc labels)
     (let loop ((labels labels) (traces '()))
@@ -152,7 +163,8 @@ that PC has one more collaterally blocked coroutine on the same port."
                           (filter-implicit-illegal-only new)))
                  (new (filter (compose not (is-status? <blocked-error>) car)
                               new))
-                 (new (filter (compose not (cute collateral? <> pc) car) new)))
+                 (new (filter (compose not (cute collateral? <> pc) car) new))
+                 (new (map (cute rewrite-trace-head prune-defer <>) new)))
             (cond
              ((find (compose (is-status? <livelock-error>) car) new)
               (format (current-error-port) "warning: livelock, bailing out\n")
@@ -160,7 +172,9 @@ that PC has one more collaterally blocked coroutine on the same port."
              (else
               (loop (cdr labels) (append new traces))))))))
 
-  (let* ((labels (cons* 'async 'external (labels)))
+  (let* ((labels (labels))
+         (labels (if (is-a? (%sut) <runtime:port>) labels
+                     (cons* 'async 'defer 'external labels)))
          (lts (make-hash-table))
          (state-number-table (make-hash-table))
          (state-number-count (list 1))
@@ -174,7 +188,6 @@ that PC has one more collaterally blocked coroutine on the same port."
       (let ((from (pc->state-number pc)))
         (unless (or (.status pc) (hash-ref lts from))
           (let* ((from-pc pc)
-                 (pc (clone pc #:collateral-instance #f))
                  (traces (run-labels pc labels))
                  (traces (map (cute append <> (list from-pc)) traces))
                  (pcs (map car traces)))
@@ -519,10 +532,10 @@ triple per label in the trail of the transition."
 (define (lts->aut lts state-count)
   "Return an LTS in Aldebaran (aut) format from LTS produced by
 RTC-LTS->LTS."
-  (format #t "des (1, ~s, ~s)\n" (length lts) state-count)
+  (format #t "des (1,~s,~s)\n" (length lts) state-count)
   (for-each (match-lambda
               ((from step to)
-               (format #t "(~s, ~s, ~s)\n" from step to)))
+               (format #t "(~s,~s,~s)\n" from step to)))
             lts))
 
 
