@@ -1,10 +1,10 @@
 ;;; Dezyne --- Dezyne command line tools
 ;;;
-;;; Copyright © 2014, 2015, 2016, 2017, 2018, 2019, 2020, 2021 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
+;;; Copyright © 2014, 2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2018, 2020, 2021 Paul Hoogendijk <paul@dezyne.org>
 ;;; Copyright © 2018 Johri van Eerd <vaneerd.johri@gmail.com>
 ;;; Copyright © 2017, 2018, 2019, 2020 Rob Wieringa <rma.wieringa@gmail.com>
-;;; Copyright © 2017, 2018, 2019 Rutger van Beusekom <rutger@dezyne.org>
+;;; Copyright © 2017, 2018, 2019, 2021, 2022 Rutger van Beusekom <rutger@dezyne.org>
 ;;;
 ;;; This file is part of Dezyne.
 ;;;
@@ -72,7 +72,9 @@
            .ast
            .behavior
            .bindings
+           .blocking?
            .column
+           .comment
            .direction
            .elements
            .else
@@ -81,7 +83,7 @@
            .event.name
            .events
            .expression
-           .external
+           .external?
            .field
            .fields
            .file-name
@@ -90,7 +92,7 @@
            .function.name
            .functions
            .ids
-           .injected
+           .injected?
            .instances
            .last?
            .left
@@ -99,16 +101,16 @@
            .location
            .message
            .name
+           .noisy?
            .offset
            .port
            .ports
            .range
-           .recursive
+           .recursive?
            .right
            .root
            .scope
            .signature
-           .silent?
            .statement
            .string
            .then
@@ -192,6 +194,7 @@
            <local>
            <location>
            <location-end>
+           <message>
            <minus>
            <model-scope>
            <model>
@@ -236,7 +239,8 @@
            <variable>
            <variables>
            <void>
-           <voidreply>)
+           <voidreply>
+           <warning>)
 
 (define-syntax define-ast
   (lambda (x)
@@ -310,6 +314,7 @@
   (length))
 
 (define-ast <locationed> (<ast>)
+  (comment)
   (location))                           ; <location>
 
 (define-ast <comment> (<locationed>)
@@ -351,7 +356,7 @@
 (define-ast <declarative-compound> (<ast-list> <declarative>))
 (define-ast <events> (<ast-list>))
 (define-ast <fields> (<ast-list>))
-(define-ast <formals> (<ast-list>))
+(define-ast <formals> (<ast-list> <scope>))
 (define-ast <functions> (<ast-list>))
 (define-ast <instances> (<ast-list>))
 (define-ast <ports> (<ast-list>))
@@ -366,8 +371,7 @@
 (define-ast <types> (<ast-list>))
 (define-ast <variables> (<ast-list>))
 
-(define-ast <import> (<named>)
-  (root))
+(define-ast <import> (<named>))
 
 (define-ast <model> (<scope> <declaration>))
 
@@ -429,9 +433,10 @@
 
 (define-ast <port> (<instance>)
   (direction)                           ; symbol 'provides / 'requires
-  (external)
+  (blocking?)
+  (external?)
   (formals #:init-form (make <formals-node>))
-  (injected))
+  (injected?))
 
 (define-ast <trigger> (<scope> <locationed>)
   (port.name)
@@ -442,9 +447,6 @@
 
 (define-ast <expression> (<locationed>))
 
-(define-ast <literal> (<expression>)
-  (value #:init-value "void"))
-
 (define-ast <binary> (<expression>)
   (left #:init-value *unspecified*)
   (right #:init-value *unspecified*))
@@ -452,6 +454,9 @@
 (define-ast <unary> (<expression>)
   (expression #:init-value *unspecified*))
 
+(define-ast <literal> (<unary>)
+  (value #:init-value "void"))
+
 (define-ast <group> (<unary>))
 
 (define-ast <bool-expr> (<expression>))
@@ -487,27 +492,27 @@
 (define-ast <data> (<data-expr>)
   (value))
 
-(define-ast <var> (<named> <expression>))
+(define-ast <var> (<named> <unary>))
 
-(define-ast <undefined> (<expression>)
+(define-ast <undefined> (<unary>)
   (name))
 
-(define-ast <variable> (<declaration> <imperative> <expression>)
+(define-ast <variable> (<declaration> <imperative> <unary>)
   (type.name)
   (expression #:init-form (make <expression-node>)))
 
-(define-ast <field-test> (<bool-expr>)
+(define-ast <field-test> (<unary> <bool-expr>)
   (variable.name)
   (field))
 
-(define-ast <enum-literal> (<enum-expr>)
+(define-ast <enum-literal> (<unary> <enum-expr>)
   (type.name)
   (field))
 
 (define-ast <otherwise> (<expression>) ;; FIXME: make <guard-otherwise/guard-else-node> instead
   (value #:init-value *unspecified*))
 
-(define-ast <formal> (<declaration> <expression>)
+(define-ast <formal> (<declaration> <unary>)
   (type.name)
   (direction #:init-value 'in))
 
@@ -537,22 +542,29 @@
 
 (define-ast <function> (<scope> <declaration>)
   (signature #:init-form (make <signature-node>))
-  (recursive)
+  (noisy?)
+  (recursive?)
   (statement))
 
-(define-ast <action> (<imperative> <expression>)
+(define-ast <action> (<imperative> <unary>)
   (port.name)
   (event.name)
   (arguments #:init-form (make <arguments-node>)))
 
-(define-ast <action-or-call> (<named> <imperative> <expression>)
+(define-ast <defer> (<scope> <imperative>)
+  (arguments)
+  (statement))
+
+(define-ast <defer-end> (<imperative>))
+
+(define-ast <action-or-call> (<named> <imperative> <unary>)
   (arguments #:init-form (make <arguments-node>)))
 
 (define-ast <assign> (<imperative>)
   (variable.name)
   (expression #:init-form (make <expression-node>)))
 
-(define-ast <call> (<imperative> <expression>)
+(define-ast <call> (<imperative> <unary>)
   (function.name)
   (arguments #:init-form (make <arguments-node>))
   (last?))
@@ -563,7 +575,7 @@
 
 (define-ast <otherwise-guard> (<guard>))
 
-(define-ast <if> (<imperative>)
+(define-ast <if> (<imperative> <scope>)
   (expression #:init-form (make <expression-node>))
   (then)
   (else))
@@ -578,8 +590,7 @@
 
 (define-ast <on> (<declarative>)
   (triggers #:init-form (make <triggers-node>))
-  (statement)
-  (silent?))
+  (statement))
 
 (define-ast <reply> (<imperative>)
   (expression #:init-form (make <literal-node>))
@@ -602,11 +613,14 @@
 (define-ast <status> (<ast>)
   (ast))
 
-(define-ast <error> (<status>)
+(define-ast <message> (<status>)
   (message #:init-value ""))
 
-(define-ast <info> (<status>)
-  (message #:init-value ""))
+(define-ast <error> (<message>))
+
+(define-ast <info> (<message>))
+
+(define-ast <warning> (<message>))
 
 (define-ast <skip> (<imperative>))
 
@@ -615,7 +629,7 @@
 (define-ast <the-end-blocking> (<statement>))
 (define-ast <voidreply> (<statement>))
 
-(define-ast <argument> (<named> <expression>)
+(define-ast <argument> (<named> <unary>)
   (type.name)
   (direction))
 
