@@ -2,7 +2,7 @@
 ;;;
 ;;; Copyright © 2018 Paul Hoogendijk <paul@dezyne.org>
 ;;; Copyright © 2018, 2019 Rob Wieringa <rma.wieringa@gmail.com>
-;;; Copyright © 2018, 2020, 2021, 2022 Rutger van Beusekom <rutger@dezyne.org>
+;;; Copyright © 2018, 2020, 2021, 2022, 2023 Rutger van Beusekom <rutger@dezyne.org>
 ;;; Copyright © 2018, 2019, 2020, 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2018 Filip Toman <filip.toman@verum.com>
 ;;;
@@ -28,7 +28,6 @@
 ;;;
 ;;; Top
 ;;;
-(define-templates header-data ast:data*)
 (define-templates header-model c++:model)
 (define-templates include-guard)
 (define-templates pump-include code:pump?)
@@ -52,7 +51,7 @@
 (define-templates injected-port-declare ast:injected-port*)
 (define-templates async-port-declare ast:async-port*)
 (define-templates declare-method code:trigger)
-(define-templates pump code:pump?)
+(define-templates pump-for-async c++:pump-for-async?)
 (define-templates method-declare code:ons)
 (define-templates declare-method code:trigger)
 (define-templates stream-member ast:variable* stream-comma-infix)
