@@ -4,7 +4,7 @@
 ;;; Copyright © 2018 Rob Wieringa <rma.wieringa@gmail.com>
 ;;; Copyright © 2018, 2019, 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2020 Johri van Eerd <vaneerd.johri@gmail.com>
-;;; Copyright © 2021 Paul Hoogendijk <paul@dezyne.org>
+;;; Copyright © 2021, 2022 Paul Hoogendijk <paul@dezyne.org>
 ;;;
 ;;; This file is part of Dezyne.
 ;;;
@@ -108,6 +108,7 @@
 (define-templates trigger-name makreel:trigger-name)
 (define-templates process-id)
 (define-templates process-identifier)
+(define-templates statement-process-identifier .statement)
 (define-templates process-parameters makreel:process-parameters parameters-grammar)
 (define-templates process-parameters-return makreel:process-parameters-return parameters-grammar)
 
@@ -120,7 +121,8 @@
 
 (define-templates continuation makreel:continuation newline-union-infix)
 (define-templates then-continuation makreel:then-continuation newline-union-infix)
-(define-templates continuation-identifier makreel:continuation) ;;ASSUME list of one
+(define-templates continuation-identifier makreel:continuation)
+(define-templates continuation-process-identifier identity)
 (define-templates else-continuation makreel:else-continuation newline-union-infix)
 
 ;; statement helpers
@@ -161,9 +163,40 @@
 (define-templates async-parallel-port ast:async-port* newline-union-prefix)
 
 ;;comonent behavior
-(define-templates component-behavior-rename-requires ast:requires+async-port* newline-comma-infix)
+(define-templates component-behavior-allow-provides ast:provides-port* newline-comma-prefix)
+(define-templates component-behavior-allow-requires ast:requires+async-port* newline-comma-prefix)
+(define-templates component-behavior-rename-requires ast:requires+async-port* newline-comma-prefix)
+
+;;defer
+(define-templates defer-semantics ast:provides-port* newline-union-infix)
+(define-templates defer (lambda (o) (if (pair? (makreel:behavior->defer-qout o)) o '())))
+(define-templates defer-qout makreel:behavior->defer-qout newline-union-infix)
+(define-templates defer-process-haakjes)
+(define-templates defer-process-argument makreel:locals comma-infix)
+(define-templates defer-proc makreel:behavior->defer-qout)
+(define-templates provides-flush (lambda (o) (if (ast:provides? o) o '())))
+(define-templates requires-reply (lambda (o) (if (ast:requires? o) o '())))
+(define-templates defer-skip (lambda (o) (if (parent o <component>) o '())))
+(define-templates defer-locals-sort (cute tree-collect (is? <defer>) <>) pipe-prefix)
+(define-templates defer-local-arguments-sort (lambda (o) (if (pair? (makreel:locals o)) o '())))
+(define-templates deferred-locals-sort makreel:locals comma-infix)
+(define-templates defer-locals)
+(define-templates defer-local-arguments (lambda (o) (if (pair? (makreel:locals o)) o '())))
+(define-templates deferred-locals makreel:locals comma-infix)
+(define-templates defer-process-index (compose makreel:process-index .statement))
+
+(define-templates state-vector (lambda (o) (let ((behavior (.behavior o))) (if (pair? (ast:variable* behavior)) behavior '()))))
+(define-templates state-member ast:variable* comma-infix)
+(define-templates construct-state-vector (lambda (o) (let ((behavior (.behavior (parent o <component>)))) (if (pair? (ast:variable* behavior)) behavior '()))))
+(define-templates state-var ast:variable* comma-infix)
+
+(define-templates defer-select-member ast:variable* pipe-prefix)
+(define-templates defer-select-variable ast:defer-variable* comma-infix)
+(define-templates defer-predicate ast:variable* and-infix)
+(define-templates defer-predicate-true (lambda (o) (if (null? (ast:variable* o)) o '())) and-infix)
 
 ;;semantics
+(define-templates semantics-main)
 (define-templates semantics-provides ast:provides-port* newline-union-infix)
 (define-templates semantics-provides-flush ast:requires-port* newline-union-prefix)
 (define-templates semantics-provides-reply-init ast:provides-port* comma-infix)
@@ -176,31 +209,35 @@
 (define-templates semantics-provides-unblocked-replies ast:provides-port* newline-union-prefix)
 (define-templates semantics-provides-unblocked-modeling ast:requires-port* newline-union-prefix)
 (define-templates semantics-provides-unblocked-switch-context ast:requires-port* newline-union-prefix)
+(define-templates semantics-async-defer)
+(define-templates semantics-async-defer-flush)
 (define-templates semantics-async ast:provides-port* newline-union-prefix)
 (define-templates semantics-async-requires ast:requires-port* newline-union-prefix)
 (define-templates semantics-async-modeling ast:requires-port* newline-union-prefix)
 (define-templates semantics-async-flush ast:provides-port* newline-union-prefix)
+(define-templates semantics-async-requires-flush ast:provides-port* newline-union-prefix)
 (define-templates semantics-async-qin ast:async-port* newline-union-infix)
 (define-templates semantics-async-qout ast:async-port* newline-union-prefix)
 (define-templates semantics-async-allow-ack (lambda (o) (let ((a (ast:async-port* o))) (if (pair? a) o '()))))
 (define-templates semantics-no-async (lambda (o) (let ((a (ast:async-port* o))) (if (pair? a) '() o))))
+(define-templates semantics-provides-blocking-defer)
 (define-templates semantics-provides-blocking-provides ast:provides-port* newline-union-prefix)
 (define-templates semantics-provides-blocking-requires ast:requires-port* newline-union-prefix)
+(define-templates semantics-provides-skip-blocked-defer)
 (define-templates semantics-provides-skip-blocked-replies ast:provides-port* newline-union-infix)
 (define-templates semantics-provides-skip-blocked-requires ast:requires-port* newline-union-prefix)
 (define-templates semantics-port-in-released-or ast:provides-port* or-infix)
+(define-templates semantics-provides-blocked-defer)
 (define-templates semantics-provides-blocked-provides ast:provides-port* newline-union-prefix)
 (define-templates semantics-provides-blocked-replies ast:provides-port* newline-union-prefix)
 (define-templates semantics-provides-blocked-ports ast:provides-port* newline-union-prefix)
 (define-templates semantics-provides-blocked-async ast:async-port* newline-union-prefix)
-(define-templates semantics-provides-blocked-qmt ast:requires-port* newline-union-prefix)
-(define-templates semantics-provides-blocked-qmt-async ast:async-port* newline-union-prefix)
 (define-templates semantics-provides-blocked-requires ast:requires-port* newline-union-prefix)
 (define-templates semantics-reply ast:provides-port* newline-union-infix)
+(define-templates semantics-blocked-rtc-defer)
 (define-templates semantics-blocked-rtc-provides ast:provides-port* newline-union-infix)
 (define-templates semantics-blocked-rtc-requires ast:requires-port* newline-union-prefix)
 (define-templates semantics-requires ast:requires-port* newline-union-prefix)
-(define-templates semantics-requires-flush ast:provides-port* newline-union-prefix)
 (define-templates semantics-comm-provides ast:provides-port* newline-comma-prefix)
 (define-templates semantics-comm-requires ast:requires+async-port* newline-comma-prefix)
 (define-templates semantics-allow-provides ast:provides-port* newline-comma-prefix)
@@ -208,6 +245,7 @@
 (define-templates semantics-allow-async ast:async-port* newline-comma-prefix)
 (define-templates semantics-rename-provides ast:provides-port* newline-comma-prefix)
 (define-templates semantics-rename-requires ast:requires+async-port* newline-comma-prefix)
+(define-templates semantics-provides-action ast:provides-port* newline-union-prefix)
 
 ;; component
 (define-templates component-comm-requires ast:requires+async-port* newline-comma-prefix)
