@@ -114,6 +114,9 @@
 (define-method (pc-event? (pc <program-counter>) (o <statement>))
   (pc-event? (.instance pc) o))
 
+(define-method (pc-event? (pc <program-counter>) (o <defer-qout>))
+  #t)
+
 (define-method (pc-event? (pc <program-counter>) (o <initial-compound>))
   (pc-event? (.instance pc) (.trigger pc)))
 
@@ -241,6 +244,9 @@
                    (trigger (trigger->string trigger)))
               (format #f "~a.~a" (trace-name r:other-port) trigger)))))))
 
+(define-method (pc->event (o <runtime:component>) (defer-qout <defer-qout>))
+  (cons defer-qout "<defer>"))
+
 (define-method (pc->event (o <runtime:component>) (q-out <q-out>))
   (let ((trigger (.trigger q-out)))
     (cons q-out (trigger->string trigger))))
@@ -316,6 +322,9 @@
 (define-method (pc-arrow? (o <runtime:component>) (action <action>))
   (not (ast:async? (.port action))))
 
+(define-method (pc-arrow? (o <runtime:component>) (action <defer-qout>))
+  #t)
+
 (define-method (pc-arrow? (o <runtime:port>) (q-out <q-out>))
   #t)
 
@@ -404,6 +413,10 @@
            (else
             (format #f "~a.~a -> ..." (runtime:instance->string r:port) (.event.name trigger)))))))
 
+(define-method (pc->arrow (o <runtime:component>) (defer-qout <defer-qout>))
+  (cons defer-qout
+        (format #f "<defer>")))
+
 (define-method (pc->arrow (o <runtime:component>) (q-in <q-in>))
   (cons q-in (format #f "~a.<q> <- ..." (runtime:instance->string o))))
 
@@ -874,7 +887,7 @@ intermediate steps such as assignments, function calls, replies,
                       (first-port-name (.name (.port first))))
                  (format #f "~aafter action on port: ~a\n" first-location first-port-name))
                (let* ((trigger (.trigger pc))
-                      (trigger-location (location-prefix (parent action <on>)))
+                      (trigger-location (location-prefix (.statement pc)))
                       (trigger-port (.port trigger))
                       (trigger-port-name (.name trigger-port)))
                  (format #f "~afor trigger on port ~a\n" trigger-location trigger-port-name))))
@@ -977,7 +990,9 @@ value of the trail."
                 (string< (string-join trail-a)
                          (string-join trail-b)))))))
 
-  (let* ((traces (sort traces trace<))
+  (let* ((error traces (partition (compose (is? <error>) .status car) traces))
+         (traces (if (null? error) traces error))
+         (traces (sort traces trace<))
          (trace-format trace)
          (trace (and (pair? traces) (car traces)))
          (pc (and trace (car trace)))
@@ -1026,7 +1041,7 @@ value of the trail."
                eligible)
       (let* ((eligible (if (and (is-a? status <end-of-trail>) (.ast status)) (end-of-trail-labels pc)
                            eligible))
-             (labels (labels pc))
+             (labels (or (and=> pc labels) '()))
              (labels (if (and (pair? eligible) (not (member (car eligible) labels))) eligible
                          labels)))
         (format #t "~s\n" (cons 'labels labels))
