@@ -3,7 +3,7 @@
 ;;; Copyright © 2019 Timothy Sample <samplet@ngyro.com>
 ;;; Copyright © 2019, 2020, 2021, 2022 Jan (janneke) Nieuwenhuizen <janneke@gnu.org>
 ;;; Copyright © 2021 Paul Hoogendijk <paul@dezyne.org>
-;;; Copyright © 2021, 2022 Rutger van Beusekom <rutger@dezyne.org>
+;;; Copyright © 2021, 2022, 2023 Rutger van Beusekom <rutger@dezyne.org>
 ;;;
 ;;; This file is part of Dezyne.
 ;;;
@@ -36,13 +36,22 @@
   #:use-module (dzn misc)
   #:use-module (dzn shell-util)
   #:use-module (dzn trace)
-  #:export (run-baseline
+  #:export (%default-stages
+            run-baseline
             run-test))
 
+(define %default-stages
+  '("verify" "traces" "simulate" "lts" "code"))
+
+
 ;;;
 ;;; Invocation helpers
 ;;;
 
+(define %interpreter-alist
+  `(("javascript" . ,(list (or (getenv "NODE") "node")))
+    ("scheme" . ,(list (or (getenv "GUILE") "guile") "--no-auto-compile"))))
+
 ;; XXX: This is probably the slowest way possible to do this.  I hope
 ;; it is correct, at least.
 (define (get-strings-all . ports)
@@ -200,9 +209,11 @@ output, and standard error as three values."
 ;;; Utility.
 ;;;
 (define (error-model? file-name)
-  (or (directory-exists? (string-append file-name "/baseline/verify"))
+  (or (file-exists? (string-append file-name "/baseline/verify.out"))
+      (file-exists? (string-append file-name "/baseline/verify.err"))
       ;; no verify baseline for system error models
-      (directory-exists? (string-append file-name "/baseline/simulate"))))
+      (file-exists? (string-append file-name "/baseline/simulate.out"))
+      (file-exists? (string-append file-name "/baseline/simulate.err"))))
 
 (define (features file-name)
   (define (feature? feature)
@@ -215,6 +226,7 @@ output, and standard error as three values."
                     (feature? "collateral"))
                 "collateral")
            (feature? "calling_context")
+           (feature? "defer")
            (and (or (feature? "multiple_out")
                     (feature? "double_inevitable")
                     (feature? "flush"))
@@ -224,7 +236,7 @@ output, and standard error as three values."
 
 (define (features-missing file-name language)
   (fold (lambda (feature missing)
-          (if (member language (assoc-ref %feature-alist feature)) '()
+          (if (member language (assoc-ref %feature-alist feature)) missing
               (cons feature missing)))
         '()
         (features file-name)))
@@ -238,12 +250,17 @@ output, and standard error as three values."
 
 (define (filter-<flush> string)
   (let* ((lines (string-split string #\newline))
-         (events (filter (negate (cut string-contains <> "<flush>")) lines)))
+         (events (filter (negate (cute string-contains <> "<flush>")) lines)))
+    (string-join events "\n")))
+
+(define (filter-<defer> string)
+  (let* ((lines (string-split string #\newline))
+         (events (filter (negate (cute string-contains <> "<defer>")) lines)))
     (string-join events "\n")))
 
 (define (filter-state string)
   (let* ((lines (string-split string #\newline))
-         (events (filter (negate (cut string-prefix? "(state " <>)) lines)))
+         (events (filter (negate (cute string-prefix? "(state " <>)) lines)))
     (string-join events "\n")))
 
 
@@ -254,13 +271,12 @@ output, and standard error as three values."
   (format #t "** stage: verify\n")
   (let* ((base-name (basename file-name))
          (dzn-name (string-append file-name "/" base-name ".dzn"))
-         (baseline (string-append file-name "/baseline/verify"))
          (includes (cons (string-append file-name "/dzn")
                          (or (and=> (getenv "DZN_INCLUDE_PATH")
-                                    (cut string-split <> #\:))
+                                    (cute string-split <> #\:))
                              '())))
          (includes (filter directory-exists? includes))
-         (includes (append-map (cut list "-I" <>) includes))
+         (includes (append-map (cute list "-I" <>) includes))
          (model (or (model? file-name) base-name))
          (queue-size (queue-size file-name))
          (command
@@ -272,7 +288,7 @@ output, and standard error as three values."
             ,dzn-name)))
     (or (skip? file-name "verify")
         (run-baseline file-name command
-                      #:baseline baseline))))
+                      #:language "verify"))))
 
 (define (run-code file-name language)
   (format #t "** stage: code: ~a\n" language)
@@ -282,12 +298,12 @@ output, and standard error as three values."
               (in-lang (string-append file-name "/" language))
               (includes (cons in-lang
                               (or (and=> (getenv "DZN_INCLUDE_PATH")
-                                         (cut string-split <> #\:))
+                                         (cute string-split <> #\:))
                                   '())))
               (includes (filter directory-exists? includes))
-              (includes (append-map (cut list "-I" <>) includes))
+              (includes (append-map (cute list "-I" <>) includes))
               (out (string-append file-name "/out"))
-              (out-lang (string-append file-name "/out/" language))
+              (out-lang (string-append out "/" language))
               (model (or (model? file-name) base-name))
               ;; FIXME: METAs `model' is used for component/system tricksery
               (model base-name))
@@ -306,9 +322,11 @@ output, and standard error as three values."
                      ,@(code-options file-name)
                      ,dzn-name) input)
                 (zero? status))
-              (let* ((dzn-files (list-files file-name ".+\\.*dzn*$"))
+              (let* ((dzn-files (append (list-files file-name ".+\\.*dzn*$")
+                                        (if (not (directory-exists? in-lang)) '()
+                                            (list-files in-lang ".+\\.*dzn*$"))))
                      (extra-dzn-files (filter
-                                       (negate (cut equal? <> dzn-name))
+                                       (negate (cute equal? <> dzn-name))
                                        dzn-files)))
                 (and-map
                  (lambda (dzn-file)
@@ -321,7 +339,7 @@ output, and standard error as three values."
                  extra-dzn-files)))))
        (run-build file-name language)
        (let ((traces (find-files file-name ".*trace.*$")))
-         (and-map (cut run-execute file-name language <>) traces))))
+         (and-map (cute run-execute file-name language <>) traces))))
 
 (define (run-traces file-name)
   (format #t "** stage: traces\n")
@@ -330,14 +348,15 @@ output, and standard error as three values."
          (input "")
          (includes (cons (string-append file-name "/dzn")
                          (or (and=> (getenv "DZN_INCLUDE_PATH")
-                                    (cut string-split <> #\:))
+                                    (cute string-split <> #\:))
                              '())))
          (includes (filter directory-exists? includes))
-         (includes (append-map (cut list "-I" <>) includes))
+         (includes (append-map (cute list "-I" <>) includes))
          (model (or (model? file-name) base-name))
          (queue-size (queue-size file-name))
+         (language "traces")
          (out (string-append file-name "/out"))
-         (out-lang (string-append file-name "/out/traces"))
+         (out-lang (string-append out "/" language))
          (handwritten-traces (list-files file-name ".*trace.*$")))
     (or (and (error-model? file-name)
              (null? handwritten-traces))
@@ -362,7 +381,7 @@ output, and standard error as three values."
          (dzn-name (string-append file-name "/" base-name ".dzn"))
          (input "")
          (out (string-append file-name "/out"))
-         (out-lang (string-append file-name "/out/" language))
+         (out-lang (string-append out "/" language))
          (thread-pool? (thread-pool? file-name))
          (command `("make"
                     "-f" ,(string-append "test/lib/build." language ".make")
@@ -386,8 +405,9 @@ output, and standard error as three values."
          (dzn-name (string-append file-name "/" base-name ".dzn"))
          (input (with-input-from-file trace read-string))
          (out (string-append file-name "/out"))
-         (out-lang (string-append file-name "/out/" language))
-         (test (string-append out-lang "/test")))
+         (out-lang (string-append out "/" language))
+         (test (string-append out-lang "/test"))
+         (interpreter (or (assoc-ref %interpreter-alist language) '())))
     (or (error-model? file-name)
         (feature-skip? file-name language)
         (skip? file-name
@@ -396,12 +416,14 @@ output, and standard error as three values."
                "build" (string-append language ":build")
                "execute" (string-append language ":execute"))
         (receive (status stdout stderr)
-            (observe `(,test ,@(if (flush? file-name) '("--flush") '())) input)
+            (observe `(,@interpreter
+                       ,test ,@(if (flush? file-name) '("--flush") '())) input)
           (and (zero? status)
                (let ((net (trace:format-trace stderr #:format "event")))
                  (receive (status stdout stderr)
                      (let* ((input (filter-state input))
-                            (input (filter-<flush> input)))
+                            (input (filter-<flush> input))
+                            (input (filter-<defer> input)))
                        (observe `("bash" "-c"
                                   ,(string-append "diff -ywB"
                                                   ;; ignore foreign/system communications
@@ -419,20 +441,20 @@ output, and standard error as three values."
          (input (with-input-from-file trace read-string))
          (includes (cons (string-append file-name "/dzn")
                          (or (and=> (getenv "DZN_INCLUDE_PATH")
-                                    (cut string-split <> #\:))
+                                    (cute string-split <> #\:))
                              '())))
          (includes (filter directory-exists? includes))
-         (includes (append-map (cut list "-I" <>) includes))
+         (includes (append-map (cute list "-I" <>) includes))
          (model (or (model? file-name) base-name))
-         (out (string-append file-name "/out"))
          (language "simulate")
-         (out-lang (string-append file-name "/out/" language))
+         (out (string-append file-name "/out"))
+         (out-lang (string-append out "/" language))
          (input (filter-<flush> input))
          (model (or (model? file-name) base-name))
          (queue-size (queue-size file-name))
          ;; FIXME: METAs `model' is used for component/system tricksery
          (model base-name)
-         (baseline? (file-exists? (string-append baseline "/" base-name)))
+         (baseline? (file-exists? (string-append baseline ".out")))
          (trace-format (cond ((trace-format file-name))
                              (baseline? "trace")
                              (else "event"))))
@@ -447,8 +469,7 @@ output, and standard error as three values."
                    ,dzn-name)
                  input)
 
-      (cond ((and (zero? status) (not (file-exists? baseline)))
-
+      (cond ((and (zero? status) (not baseline?))
              (let* ((input (filter-state input))
                     (trail (filter-state stdout)))
                (and
@@ -461,29 +482,29 @@ output, and standard error as three values."
                              #f)
                   (zero? status))
                 (or (string-null? stderr)
-                    (and (format (current-error-port) "ERROR: ~a\n" stderr)
-                         #f)))))
+                    (format (current-error-port) "ERROR: ~a\n" stderr)))))
             (else
              (let ((out-file (string-append out-lang "/out"))
                    (err-file (string-append out-lang "/err")))
                (mkdir-p out-lang)
                (with-output-to-file out-file
-                 (cut display stdout))
+                 (cute display stdout))
                (with-output-to-file err-file
-                 (cut display stderr))
-               (let ((baseline-out (string-append baseline "/" base-name))
-                     (baseline-err (string-append baseline "/" base-name ".stderr")))
+                 (cute display stderr))
+               (let ((baseline-out (string-append baseline ".out"))
+                     (baseline-err (string-append baseline ".err")))
                  (and (zero? (system* "diff" "-ywB" baseline-out out-file))
                       (or (and (string-null? stderr)
                                (not (file-exists? baseline-err)))
-                          (zero? (system* "diff" "-ywB" baseline-err err-file)))))))))))
+                          (zero? (system* "diff" "-ywB"
+                                          baseline-err err-file)))))))))))
 
 (define (run-simulate file-name)
   (format #t "** stage: simulate\n")
   (or (skip? file-name
              "simulate")
       (let ((traces (find-files file-name ".*trace.*$")))
-        (and-map (cut run-simulate-trace file-name <>) traces))))
+        (and-map (cute run-simulate-trace file-name <>) traces))))
 
 (define (run-lts file-name)
   (format #t "** stage: lts\n")
@@ -492,14 +513,14 @@ output, and standard error as three values."
          (input "")
          (includes (cons (string-append file-name "/dzn")
                          (or (and=> (getenv "DZN_INCLUDE_PATH")
-                                    (cut string-split <> #\:))
+                                    (cute string-split <> #\:))
                              '())))
-         (includes (append-map (cut list "-I" <>) includes))
+         (includes (append-map (cute list "-I" <>) includes))
          (model (or (model? file-name) base-name))
          (queue-size (queue-size file-name))
-         (out (string-append file-name "/out"))
          (language "lts")
-         (out-lang (string-append file-name "/out/" language))
+         (out (string-append file-name "/out"))
+         (out-lang (string-append out "/" language))
          (handwritten-traces (list-files file-name ".*trace.*$")))
     (or (and (error-model? file-name)
              (null? handwritten-traces))
@@ -553,7 +574,7 @@ output, and standard error as three values."
                               "")
                            (display stdout)
                            (let* ((lines (and stdout (string-split stdout #\newline)))
-                                  (stdout-status (and lines (filter (cut string-prefix? "result: " <>) lines)))
+                                  (stdout-status (and lines (filter (cute string-prefix? "result: " <>) lines)))
                                   (stdout-status (and (pair? stdout-status) (car stdout-status)))
                                   (status (if (and (zero? status)
                                                    stdout-status
@@ -568,20 +589,25 @@ output, and standard error as three values."
 (define* (run-baseline file-name command #:key
                        (baseline (string-append file-name "/baseline"))
                        (input "")
+                       (language "parse")
                        (stdout-filter identity))
-  (receive (status stdout stderr)
-      (observe command input)
-    (or (and (zero? status)
-             (not (directory-exists? baseline)))
-        (let ((out-file (string-append file-name ".out"))
-              (err-file (string-append file-name ".err")))
-          (with-output-to-file out-file
-            (cut display (stdout-filter stdout)))
-          (with-output-to-file err-file
-            (cut display stderr))
-          (let* ((base-name (basename file-name))
-                 (baseline-out (string-append baseline "/" base-name))
-                 (baseline-err (string-append baseline-out ".stderr")))
+  (let* ((base-out (string-append baseline "/" language))
+         (baseline-out (string-append base-out ".out"))
+         (baseline-err (string-append base-out ".err")))
+    (receive (status stdout stderr)
+        (observe command input)
+      (or (and (zero? status)
+               (not (file-exists? baseline-out))
+               (not (file-exists? baseline-err)))
+          (let* ((out (string-append file-name "/out"))
+                 (out-lang (string-append out "/" language))
+                 (out-file (string-append out-lang "/out"))
+                 (err-file (string-append out-lang "/err")))
+            (mkdir-p out-lang)
+            (with-output-to-file out-file
+              (cute display (stdout-filter stdout)))
+            (with-output-to-file err-file
+              (cute display stderr))
             (and (or (and (string-null? stdout)
                           (not (file-exists? baseline-out)))
                      (zero? (system* "diff" "-uwB" baseline-out out-file)))
@@ -590,14 +616,18 @@ output, and standard error as three values."
                      (zero? (system* "diff" "-uwB"
                                      baseline-err err-file)))))))))
 
-(define (run-test file-name languages)
+(define* (run-test file-name languages #:key (stages %default-stages))
+  (define (skip? stage)
+    (not (member stage stages)))
   (setvbuf (current-output-port) 'line)
   (format #t "* run: ~a ~a\n" file-name languages)
-  (let ((result (and (run-verify file-name)
-                     (run-traces file-name)
-                     (run-lts file-name)
-                     (run-simulate file-name)
-                     (and-map (cut run-code file-name <>) languages))))
+  (let ((result
+         (and
+          (or (skip? "verify") (run-verify file-name))
+          (or (skip? "traces") (run-traces file-name))
+          (or (skip? "simulate") (run-simulate file-name))
+          (or (skip? "lts") (run-lts file-name))
+          (or (skip? "code") (and-map (cute run-code file-name <>) languages)))))
     (format #t "# Local Variables:
 # mode: org
 # End:
